"""
Test behaviors specific to importlib_metadata.

These tests are excluded downstream in CPython as they
test functionality only in importlib_metadata or require
behaviors ('packaging') that aren't available in the
stdlib.
"""

import unittest

import packaging.requirements
import packaging.version

from importlib_metadata import (
    _compat,
    version,
)

from . import fixtures


class IntegrationTests(fixtures.DistInfoPkg, unittest.TestCase):
    def test_package_spec_installed(self):
        """
        Illustrate the recommended procedure to determine if
        a specified version of a package is installed.
        """

        def is_installed(package_spec):
            req = packaging.requirements.Requirement(package_spec)
            return version(req.name) in req.specifier

        assert is_installed('distinfo-pkg==1.0')
        assert is_installed('distinfo-pkg>=1.0,<2.0')
        assert not is_installed('distinfo-pkg<1.0')


class FinderTests(fixtures.Fixtures, unittest.TestCase):
    def test_finder_without_module(self):
        class ModuleFreeFinder:
            """
            A finder without an __module__ attribute
            """

            def find_module(self, name):
                pass

            def __getattribute__(self, name):
                if name == '__module__':
                    raise AttributeError(name)
                return super().__getattribute__(name)

        self.fixtures.enter_context(fixtures.install_finder(ModuleFreeFinder()))
        _compat.disable_stdlib_finder()



import sys

collect_ignore = [
    # this module fails mypy tests because 'setup.py' matches './setup.py'
    'tests/data/sources/example/setup.py',
]


def pytest_configure():
    remove_importlib_metadata()


def remove_importlib_metadata():
    """
    Ensure importlib_metadata is not imported yet.

    Because pytest or other modules might import
    importlib_metadata, the coverage reports are broken (#322).
    Work around the issue by undoing the changes made by a
    previous import of importlib_metadata (if any).
    """
    sys.meta_path[:] = [
        item
        for item in sys.meta_path
        if item.__class__.__name__ != 'MetadataPathFinder'
    ]
    for mod in list(sys.modules):
        if mod.startswith('importlib_metadata'):
            del sys.modules[mod]



from jaraco.test.cpython import from_test_support, try_import

os_helper = try_import('os_helper') or from_test_support(
    'FS_NONASCII', 'skip_unless_symlink', 'temp_dir'
)
import_helper = try_import('import_helper') or from_test_support(
    'modules_setup', 'modules_cleanup'
)
